@@ -3,8 +3,8 @@ use std::borrow::Cow;
 use std::fmt;
 
 use pyo3::exceptions::{PyKeyError, PyTypeError};
-use pyo3::once_cell::GILOnceCell;
 use pyo3::prelude::*;
+use pyo3::sync::GILOnceCell;
 use pyo3::types::{PyDict, PyList};
 
 use ahash::AHashMap;
@@ -18,11 +18,11 @@ use crate::tools::{extract_i64, py_err, py_error_type};
 use super::PydanticCustomError;
 
 #[pyfunction]
-pub fn list_all_errors(py: Python) -> PyResult<&PyList> {
-    let mut errors: Vec<&PyDict> = Vec::with_capacity(100);
+pub fn list_all_errors(py: Python) -> PyResult<Bound<'_, PyList>> {
+    let mut errors: Vec<Bound<'_, PyDict>> = Vec::with_capacity(100);
     for error_type in ErrorType::iter() {
         if !matches!(error_type, ErrorType::CustomError { .. }) {
-            let d = PyDict::new(py);
+            let d = PyDict::new_bound(py);
             d.set_item("type", error_type.to_string())?;
             let message_template_python = error_type.message_template_python();
             d.set_item("message_template_python", message_template_python)?;
@@ -39,11 +39,11 @@ pub fn list_all_errors(py: Python) -> PyResult<&PyList> {
             errors.push(d);
         }
     }
-    Ok(PyList::new(py, errors))
+    Ok(PyList::new_bound(py, errors))
 }
 
 fn field_from_context<'py, T: FromPyObject<'py>>(
-    context: Option<&'py PyDict>,
+    context: Option<&Bound<'py, PyDict>>,
     field_name: &str,
     enum_name: &str,
     type_name_fn: fn() -> &'static str,
@@ -56,15 +56,12 @@ fn field_from_context<'py, T: FromPyObject<'py>>(
         .map_err(|_| py_error_type!(PyTypeError; "{}: '{}' context value must be a {}", enum_name, field_name, type_name_fn()))
 }
 
-fn cow_field_from_context<'py, T: FromPyObject<'py>, B: ?Sized + 'static>(
-    context: Option<&'py PyDict>,
+fn cow_field_from_context<'py, T: FromPyObject<'py>, B: ToOwned<Owned = T> + ?Sized + 'static>(
+    context: Option<&Bound<'py, PyDict>>,
     field_name: &str,
     enum_name: &str,
     _type_name_fn: fn() -> &'static str,
-) -> PyResult<Cow<'static, B>>
-where
-    B: ToOwned<Owned = T>,
-{
+) -> PyResult<Cow<'static, B>> {
     let res: T = field_from_context(context, field_name, enum_name, || {
         type_name::<T>().split("::").last().unwrap()
     })?;
@@ -101,7 +98,7 @@ macro_rules! error_types {
             ),+,
         }
         impl ErrorType {
-            pub fn new(py: Python, value: &str, context: Option<&PyDict>) -> PyResult<Self> {
+            pub fn new(py: Python, value: &str, context: Option<Bound<'_, PyDict>>) -> PyResult<Self> {
                 let lookup = ERROR_TYPE_LOOKUP.get_or_init(py, Self::build_lookup);
                 let error_type = match lookup.get(value) {
                     Some(error_type) => error_type.clone(),
@@ -111,17 +108,18 @@ macro_rules! error_types {
                     $(
                         Self::$item { .. } => {
                             Ok(Self::$item {
-                                context: context.map(|c| c.into_py(py)),
                                 $(
-                                    $key: $ctx_fn(context, stringify!($key), stringify!($item), || stringify!($ctx_type))?,
+                                    $key: $ctx_fn(context.as_ref(), stringify!($key), stringify!($item), || stringify!($ctx_type))?,
                                 )*
+                                context: context.map(|c| c.unbind()),
                             })
                         },
                     )+
                 }
             }
 
-            fn py_dict_update_ctx(&self, py: Python, dict: &PyDict) -> PyResult<bool> {
+            fn py_dict_update_ctx(&self, py: Python, dict: &Bound<'_, PyDict>) -> PyResult<bool> {
+                use pyo3::types::PyMapping;
                 match self {
                     $(
                         Self::$item { context, $($key,)* } => {
@@ -129,7 +127,7 @@ macro_rules! error_types {
                                 dict.set_item::<&str, Py<PyAny>>(stringify!($key), $key.to_object(py))?;
                             )*
                             if let Some(ctx) = context {
-                                dict.update(ctx.as_ref(py).downcast()?)?;
+                                dict.update(ctx.bind(py).downcast::<PyMapping>()?)?;
                                 Ok(true)
                             } else {
                                 Ok(false)
@@ -461,8 +459,8 @@ static ERROR_TYPE_LOOKUP: GILOnceCell<AHashMap<String, ErrorType>> = GILOnceCell
 impl ErrorType {
     pub fn new_custom_error(py: Python, custom_error: PydanticCustomError) -> Self {
         Self::CustomError {
-            error_type: custom_error.error_type(),
-            message_template: custom_error.message_template(),
+            error_type: custom_error.error_type().to_owned(),
+            message_template: custom_error.message_template().to_owned(),
             context: custom_error.context(py),
         }
     }
@@ -669,20 +667,20 @@ impl ErrorType {
             Self::ValueError { error, .. } => {
                 let error = &error
                     .as_ref()
-                    .map_or(Cow::Borrowed("None"), |v| Cow::Owned(v.as_ref(py).to_string()));
+                    .map_or(Cow::Borrowed("None"), |v| Cow::Owned(v.bind(py).to_string()));
                 render!(tmpl, error)
             }
             Self::AssertionError { error, .. } => {
                 let error = &error
                     .as_ref()
-                    .map_or(Cow::Borrowed("None"), |v| Cow::Owned(v.as_ref(py).to_string()));
+                    .map_or(Cow::Borrowed("None"), |v| Cow::Owned(v.bind(py).to_string()));
                 render!(tmpl, error)
             }
             Self::CustomError {
                 message_template,
                 context,
                 ..
-            } => PydanticCustomError::format_message(message_template, context.as_ref().map(|c| c.as_ref(py))),
+            } => PydanticCustomError::format_message(message_template, context.as_ref().map(|c| c.bind(py))),
             Self::LiteralError { expected, .. } => render!(tmpl, expected),
             Self::DateParsing { error, .. } => render!(tmpl, error),
             Self::DateFromDatetimeParsing { error, .. } => render!(tmpl, error),
@@ -729,8 +727,8 @@ impl ErrorType {
     }
 
     pub fn py_dict(&self, py: Python) -> PyResult<Option<Py<PyDict>>> {
-        let dict = PyDict::new(py);
-        let custom_ctx_used = self.py_dict_update_ctx(py, dict)?;
+        let dict = PyDict::new_bound(py);
+        let custom_ctx_used = self.py_dict_update_ctx(py, &dict)?;
 
         if let Self::CustomError { .. } = self {
             if custom_ctx_used {
@@ -785,7 +783,7 @@ impl From<Int> for Number {
 }
 
 impl FromPyObject<'_> for Number {
-    fn extract(obj: &PyAny) -> PyResult<Self> {
+    fn extract_bound(obj: &Bound<'_, PyAny>) -> PyResult<Self> {
         if let Some(int) = extract_i64(obj) {
             Ok(Number::Int(int))
         } else if let Ok(float) = obj.extract::<f64>() {
