@@ -1,11 +1,12 @@
 use std::fmt::Debug;
 
 use enum_dispatch::enum_dispatch;
+use jiter::StringCacheMode;
 
 use pyo3::exceptions::PyTypeError;
-use pyo3::once_cell::GILOnceCell;
 use pyo3::prelude::*;
-use pyo3::types::{PyAny, PyDict, PyTuple, PyType};
+use pyo3::sync::GILOnceCell;
+use pyo3::types::{PyAny, PyDict, PyString, PyTuple, PyType};
 use pyo3::{intern, PyTraverseError, PyVisit};
 
 use crate::build_tools::{py_schema_err, py_schema_error_type, SchemaError};
@@ -30,6 +31,7 @@ mod datetime;
 pub(crate) mod decimal;
 mod definitions;
 mod dict;
+mod enum_;
 mod float;
 mod frozenset;
 mod function;
@@ -76,7 +78,7 @@ impl PySome {
 #[pymethods]
 impl PySome {
     pub fn __repr__(&self, py: Python) -> PyResult<String> {
-        Ok(format!("Some({})", self.value.as_ref(py).repr()?,))
+        Ok(format!("Some({})", self.value.bind(py).repr()?,))
     }
 
     #[new]
@@ -86,13 +88,13 @@ impl PySome {
 
     #[classmethod]
     #[pyo3(signature = (_item, /))]
-    pub fn __class_getitem__(cls: &PyType, _item: &PyAny) -> Py<PyType> {
-        cls.into_py(cls.py())
+    pub fn __class_getitem__(cls: Py<PyType>, _item: &Bound<'_, PyAny>) -> Py<PyType> {
+        cls
     }
 
     #[classattr]
-    fn __match_args__(py: Python) -> &PyTuple {
-        PyTuple::new(py, vec![intern!(py, "value")])
+    fn __match_args__(py: Python) -> Bound<'_, PyTuple> {
+        PyTuple::new_bound(py, vec![intern!(py, "value")])
     }
 }
 
@@ -109,19 +111,21 @@ pub struct SchemaValidator {
     title: PyObject,
     hide_input_in_errors: bool,
     validation_error_cause: bool,
+    cache_str: StringCacheMode,
 }
 
 #[pymethods]
 impl SchemaValidator {
     #[new]
-    pub fn py_new(py: Python, schema: &PyAny, config: Option<&PyDict>) -> PyResult<Self> {
+    #[pyo3(signature = (schema, config=None))]
+    pub fn py_new(py: Python, schema: &Bound<'_, PyAny>, config: Option<&Bound<'_, PyDict>>) -> PyResult<Self> {
         let mut definitions_builder = DefinitionsBuilder::new();
 
         let validator = build_validator(schema, config, &mut definitions_builder)?;
         let definitions = definitions_builder.finish()?;
         let py_schema = schema.into_py(py);
         let py_config = match config {
-            Some(c) if !c.is_empty() => Some(c.into_py(py)),
+            Some(c) if !c.is_empty() => Some(c.clone().into()),
             _ => None,
         };
         let config_title = match config {
@@ -134,6 +138,9 @@ impl SchemaValidator {
         };
         let hide_input_in_errors: bool = config.get_as(intern!(py, "hide_input_in_errors"))?.unwrap_or(false);
         let validation_error_cause: bool = config.get_as(intern!(py, "validation_error_cause"))?.unwrap_or(false);
+        let cache_str: StringCacheMode = config
+            .get_as(intern!(py, "cache_strings"))?
+            .unwrap_or(StringCacheMode::All);
         Ok(Self {
             validator,
             definitions,
@@ -142,10 +149,11 @@ impl SchemaValidator {
             title,
             hide_input_in_errors,
             validation_error_cause,
+            cache_str,
         })
     }
 
-    pub fn __reduce__(slf: &PyCell<Self>) -> PyResult<(PyObject, (PyObject, PyObject))> {
+    pub fn __reduce__(slf: &Bound<Self>) -> PyResult<(PyObject, (PyObject, PyObject))> {
         // Enables support for `pickle` serialization.
         let py = slf.py();
         let cls = slf.get_type().into();
@@ -157,11 +165,11 @@ impl SchemaValidator {
     pub fn validate_python(
         &self,
         py: Python,
-        input: &PyAny,
+        input: &Bound<'_, PyAny>,
         strict: Option<bool>,
         from_attributes: Option<bool>,
-        context: Option<&PyAny>,
-        self_instance: Option<&PyAny>,
+        context: Option<&Bound<'_, PyAny>>,
+        self_instance: Option<&Bound<'_, PyAny>>,
     ) -> PyResult<PyObject> {
         self._validate(
             py,
@@ -179,11 +187,11 @@ impl SchemaValidator {
     pub fn isinstance_python(
         &self,
         py: Python,
-        input: &PyAny,
+        input: &Bound<'_, PyAny>,
         strict: Option<bool>,
         from_attributes: Option<bool>,
-        context: Option<&PyAny>,
-        self_instance: Option<&PyAny>,
+        context: Option<&Bound<'_, PyAny>>,
+        self_instance: Option<&Bound<'_, PyAny>>,
     ) -> PyResult<bool> {
         match self._validate(
             py,
@@ -206,10 +214,10 @@ impl SchemaValidator {
     pub fn validate_json(
         &self,
         py: Python,
-        input: &PyAny,
+        input: &Bound<'_, PyAny>,
         strict: Option<bool>,
-        context: Option<&PyAny>,
-        self_instance: Option<&PyAny>,
+        context: Option<&Bound<'_, PyAny>>,
+        self_instance: Option<&Bound<'_, PyAny>>,
     ) -> PyResult<PyObject> {
         let r = match json::validate_json_bytes(input) {
             Ok(v_match) => self._validate_json(
@@ -229,9 +237,9 @@ impl SchemaValidator {
     pub fn validate_strings(
         &self,
         py: Python,
-        input: &PyAny,
+        input: Bound<'_, PyAny>,
         strict: Option<bool>,
-        context: Option<&PyAny>,
+        context: Option<&Bound<'_, PyAny>>,
     ) -> PyResult<PyObject> {
         let t = InputType::String;
         let string_mapping = StringMapping::new_value(input).map_err(|e| self.prepare_validation_err(py, e, t))?;
@@ -247,12 +255,12 @@ impl SchemaValidator {
     pub fn validate_assignment(
         &self,
         py: Python,
-        obj: &PyAny,
+        obj: Bound<'_, PyAny>,
         field_name: &str,
-        field_value: &PyAny,
+        field_value: Bound<'_, PyAny>,
         strict: Option<bool>,
         from_attributes: Option<bool>,
-        context: Option<&PyAny>,
+        context: Option<&Bound<'_, PyAny>>,
     ) -> PyResult<PyObject> {
         let extra = Extra {
             input_type: InputType::Python,
@@ -261,17 +269,23 @@ impl SchemaValidator {
             from_attributes,
             context,
             self_instance: None,
+            cache_str: self.cache_str,
         };
 
         let guard = &mut RecursionState::default();
         let mut state = ValidationState::new(extra, guard);
         self.validator
-            .validate_assignment(py, obj, field_name, field_value, &mut state)
+            .validate_assignment(py, &obj, field_name, &field_value, &mut state)
             .map_err(|e| self.prepare_validation_err(py, e, InputType::Python))
     }
 
     #[pyo3(signature = (*, strict=None, context=None))]
-    pub fn get_default_value(&self, py: Python, strict: Option<bool>, context: Option<&PyAny>) -> PyResult<PyObject> {
+    pub fn get_default_value(
+        &self,
+        py: Python,
+        strict: Option<bool>,
+        context: Option<&Bound<'_, PyAny>>,
+    ) -> PyResult<PyObject> {
         let extra = Extra {
             input_type: InputType::Python,
             data: None,
@@ -279,6 +293,7 @@ impl SchemaValidator {
             from_attributes: None,
             context,
             self_instance: None,
+            cache_str: self.cache_str,
         };
         let recursion_guard = &mut RecursionState::default();
         let mut state = ValidationState::new(extra, recursion_guard);
@@ -294,10 +309,15 @@ impl SchemaValidator {
 
     pub fn __repr__(&self, py: Python) -> String {
         format!(
-            "SchemaValidator(title={:?}, validator={:#?}, definitions={:#?})",
+            "SchemaValidator(title={:?}, validator={:#?}, definitions={:#?}, cache_strings={})",
             self.title.extract::<&str>(py).unwrap(),
             self.validator,
             self.definitions,
+            match self.cache_str {
+                StringCacheMode::All => "True",
+                StringCacheMode::Keys => "'keys'",
+                StringCacheMode::None => "False",
+            }
         )
     }
 
@@ -313,22 +333,26 @@ impl SchemaValidator {
 
 impl SchemaValidator {
     #[allow(clippy::too_many_arguments)]
-    fn _validate<'s, 'data>(
-        &'data self,
-        py: Python<'data>,
-        input: &'data impl Input<'data>,
+    fn _validate<'py>(
+        &self,
+        py: Python<'py>,
+        input: &(impl Input<'py> + ?Sized),
         input_type: InputType,
         strict: Option<bool>,
         from_attributes: Option<bool>,
-        context: Option<&'data PyAny>,
-        self_instance: Option<&PyAny>,
-    ) -> ValResult<PyObject>
-    where
-        's: 'data,
-    {
+        context: Option<&Bound<'py, PyAny>>,
+        self_instance: Option<&Bound<'py, PyAny>>,
+    ) -> ValResult<PyObject> {
         let mut recursion_guard = RecursionState::default();
         let mut state = ValidationState::new(
-            Extra::new(strict, from_attributes, context, self_instance, input_type),
+            Extra::new(
+                strict,
+                from_attributes,
+                context,
+                self_instance,
+                input_type,
+                self.cache_str,
+            ),
             &mut recursion_guard,
         );
         self.validator.validate(py, input, &mut state)
@@ -337,11 +361,11 @@ impl SchemaValidator {
     fn _validate_json(
         &self,
         py: Python,
-        input: &PyAny,
+        input: &Bound<'_, PyAny>,
         json_data: &[u8],
         strict: Option<bool>,
-        context: Option<&PyAny>,
-        self_instance: Option<&PyAny>,
+        context: Option<&Bound<'_, PyAny>>,
+        self_instance: Option<&Bound<'_, PyAny>>,
     ) -> ValResult<PyObject> {
         let json_value =
             jiter::JsonValue::parse(json_data, true).map_err(|e| json::map_json_err(input, e, json_data))?;
@@ -377,27 +401,28 @@ impl<'py> SelfValidator<'py> {
         Ok(Self { validator })
     }
 
-    pub fn validate_schema(&self, py: Python<'py>, schema: &'py PyAny, strict: Option<bool>) -> PyResult<&'py PyAny> {
+    pub fn validate_schema(&self, schema: &Bound<'py, PyAny>, strict: Option<bool>) -> PyResult<Bound<'py, PyAny>> {
+        let py = schema.py();
         let mut recursion_guard = RecursionState::default();
         let mut state = ValidationState::new(
-            Extra::new(strict, None, None, None, InputType::Python),
+            Extra::new(strict, None, None, None, InputType::Python, true.into()),
             &mut recursion_guard,
         );
         match self.validator.validator.validate(py, schema, &mut state) {
-            Ok(schema_obj) => Ok(schema_obj.into_ref(py)),
+            Ok(schema_obj) => Ok(schema_obj.into_bound(py)),
             Err(e) => Err(SchemaError::from_val_error(py, e)),
         }
     }
 
     fn build(py: Python) -> PyResult<SchemaValidator> {
         let code = include_str!("../self_schema.py");
-        let locals = PyDict::new(py);
-        py.run(code, None, Some(locals))?;
-        let self_schema: &PyDict = locals.get_as_req(intern!(py, "self_schema"))?;
+        let locals = PyDict::new_bound(py);
+        py.run_bound(code, None, Some(&locals))?;
+        let self_schema = locals.get_as_req(intern!(py, "self_schema"))?;
 
         let mut definitions_builder = DefinitionsBuilder::new();
 
-        let validator = match build_validator(self_schema, None, &mut definitions_builder) {
+        let validator = match build_validator(&self_schema, None, &mut definitions_builder) {
             Ok(v) => v,
             Err(err) => return py_schema_err!("Error building self-schema:\n  {}", err),
         };
@@ -410,14 +435,15 @@ impl<'py> SelfValidator<'py> {
             title: "Self Schema".into_py(py),
             hide_input_in_errors: false,
             validation_error_cause: false,
+            cache_str: true.into(),
         })
     }
 }
 
 #[pyfunction(signature = (schema, *, strict = None))]
-pub fn validate_core_schema<'a>(py: Python<'a>, schema: &'a PyAny, strict: Option<bool>) -> PyResult<&'a PyAny> {
-    let self_validator = SelfValidator::new(py)?;
-    self_validator.validate_schema(py, schema, strict)
+pub fn validate_core_schema<'py>(schema: &Bound<'py, PyAny>, strict: Option<bool>) -> PyResult<Bound<'py, PyAny>> {
+    let self_validator = SelfValidator::new(schema.py())?;
+    self_validator.validate_schema(schema, strict)
 }
 
 pub trait BuildValidator: Sized {
@@ -426,17 +452,17 @@ pub trait BuildValidator: Sized {
     /// Build a new validator from the schema, the return type is a trait to provide a way for validators
     /// to return other validators, see `string.rs`, `int.rs`, `float.rs` and `function.rs` for examples
     fn build(
-        schema: &PyDict,
-        config: Option<&PyDict>,
+        schema: &Bound<'_, PyDict>,
+        config: Option<&Bound<'_, PyDict>>,
         definitions: &mut DefinitionsBuilder<CombinedValidator>,
     ) -> PyResult<CombinedValidator>;
 }
 
 /// Logic to create a particular validator, called in the `validator_match` macro, then in turn by `build_validator`
-fn build_specific_validator<'a, T: BuildValidator>(
+fn build_specific_validator<T: BuildValidator>(
     val_type: &str,
-    schema_dict: &'a PyDict,
-    config: Option<&'a PyDict>,
+    schema_dict: &Bound<'_, PyDict>,
+    config: Option<&Bound<'_, PyDict>>,
     definitions: &mut DefinitionsBuilder<CombinedValidator>,
 ) -> PyResult<CombinedValidator> {
     T::build(schema_dict, config, definitions)
@@ -455,13 +481,14 @@ macro_rules! validator_match {
     };
 }
 
-pub fn build_validator<'a>(
-    schema: &'a PyAny,
-    config: Option<&'a PyDict>,
+pub fn build_validator(
+    schema: &Bound<'_, PyAny>,
+    config: Option<&Bound<'_, PyDict>>,
     definitions: &mut DefinitionsBuilder<CombinedValidator>,
 ) -> PyResult<CombinedValidator> {
-    let dict: &PyDict = schema.downcast()?;
-    let type_: &str = dict.get_as_req(intern!(schema.py(), "type"))?;
+    let dict = schema.downcast::<PyDict>()?;
+    let type_: Bound<'_, PyString> = dict.get_as_req(intern!(schema.py(), "type"))?;
+    let type_ = type_.to_str()?;
     validator_match!(
         type_,
         dict,
@@ -509,6 +536,8 @@ pub fn build_validator<'a>(
         call::CallValidator,
         // literals
         literal::LiteralValidator,
+        // enums
+        enum_::BuildEnumValidator,
         // any
         any::AnyValidator,
         // bytes
@@ -556,29 +585,32 @@ pub fn build_validator<'a>(
 
 /// More (mostly immutable) data to pass between validators, should probably be class `Context`,
 /// but that would confuse it with context as per pydantic/pydantic#1549
-#[derive(Debug)]
-pub struct Extra<'a> {
+#[derive(Debug, Clone)]
+pub struct Extra<'a, 'py> {
     /// Validation mode
     pub input_type: InputType,
     /// This is used as the `data` kwargs to validator functions
-    pub data: Option<&'a PyDict>,
+    pub data: Option<Bound<'py, PyDict>>,
     /// whether we're in strict or lax mode
     pub strict: Option<bool>,
     /// Validation time setting of `from_attributes`
     pub from_attributes: Option<bool>,
     /// context used in validator functions
-    pub context: Option<&'a PyAny>,
+    pub context: Option<&'a Bound<'py, PyAny>>,
     /// This is an instance of the model or dataclass being validated, when validation is performed from `__init__`
-    self_instance: Option<&'a PyAny>,
+    self_instance: Option<&'a Bound<'py, PyAny>>,
+    /// Whether to use a cache of short strings to accelerate python string construction
+    cache_str: StringCacheMode,
 }
 
-impl<'a> Extra<'a> {
+impl<'a, 'py> Extra<'a, 'py> {
     pub fn new(
         strict: Option<bool>,
         from_attributes: Option<bool>,
-        context: Option<&'a PyAny>,
-        self_instance: Option<&'a PyAny>,
+        context: Option<&'a Bound<'py, PyAny>>,
+        self_instance: Option<&'a Bound<'py, PyAny>>,
         input_type: InputType,
+        cache_str: StringCacheMode,
     ) -> Self {
         Extra {
             input_type,
@@ -587,19 +619,21 @@ impl<'a> Extra<'a> {
             from_attributes,
             context,
             self_instance,
+            cache_str,
         }
     }
 }
 
-impl<'a> Extra<'a> {
+impl Extra<'_, '_> {
     pub fn as_strict(&self) -> Self {
         Self {
             input_type: self.input_type,
-            data: self.data,
+            data: self.data.clone(),
             strict: Some(true),
             from_attributes: self.from_attributes,
             context: self.context,
             self_instance: self.self_instance,
+            cache_str: self.cache_str,
         }
     }
 }
@@ -652,6 +686,11 @@ pub enum CombinedValidator {
     FunctionCall(call::CallValidator),
     // literals
     Literal(literal::LiteralValidator),
+    // enums
+    IntEnum(enum_::EnumValidator<enum_::IntEnumValidator>),
+    StrEnum(enum_::EnumValidator<enum_::StrEnumValidator>),
+    FloatEnum(enum_::EnumValidator<enum_::FloatEnumValidator>),
+    PlainEnum(enum_::EnumValidator<enum_::PlainEnumValidator>),
     // any
     Any(any::AnyValidator),
     // bytes
@@ -701,32 +740,32 @@ pub enum CombinedValidator {
 #[enum_dispatch(CombinedValidator)]
 pub trait Validator: Send + Sync + Debug {
     /// Do the actual validation for this schema/type
-    fn validate<'data>(
+    fn validate<'py>(
         &self,
-        py: Python<'data>,
-        input: &'data impl Input<'data>,
-        state: &mut ValidationState,
+        py: Python<'py>,
+        input: &(impl Input<'py> + ?Sized),
+        state: &mut ValidationState<'_, 'py>,
     ) -> ValResult<PyObject>;
 
     /// Get a default value, currently only used by `WithDefaultValidator`
-    fn default_value(
+    fn default_value<'py>(
         &self,
-        _py: Python<'_>,
+        _py: Python<'py>,
         _outer_loc: Option<impl Into<LocItem>>,
-        _state: &mut ValidationState,
+        _state: &mut ValidationState<'_, 'py>,
     ) -> ValResult<Option<PyObject>> {
         Ok(None)
     }
 
     /// Validate assignment to a field of a model
     #[allow(clippy::too_many_arguments)]
-    fn validate_assignment<'data>(
+    fn validate_assignment<'py>(
         &self,
-        _py: Python<'data>,
-        _obj: &'data PyAny,
-        _field_name: &'data str,
-        _field_value: &'data PyAny,
-        _state: &mut ValidationState,
+        _py: Python<'py>,
+        _obj: &Bound<'py, PyAny>,
+        _field_name: &str,
+        _field_value: &Bound<'py, PyAny>,
+        _state: &mut ValidationState<'_, 'py>,
     ) -> ValResult<PyObject> {
         let py_err = PyTypeError::new_err(format!("validate_assignment is not supported for {}", self.get_name()));
         Err(py_err.into())
